/* Sphere.h declares a Sphere class.
 * Joel C. Adams, Spring 2001
 * for C++ An Introduction to Computing 3/e
 */


#ifndef SPHERE
#define SPHERE

#include <iostream>   // cerr, istream, ostream, ...
#include <cmath>      // pow()
using namespace std;

class Sphere
{
 public:
         Sphere();

  double getRadius() const;
  double getDensity() const;
  double getWeight() const;

  void   setRadiusAndDensity(double newRadius, double newDensity);
  void   setRadiusAndWeight(double newRadius, double newWeight);
  void   setDensityAndWeight(double newDensity, double newWeight);

  void   print(ostream & out) const;
  friend ostream& operator<<(ostream& out, const Sphere& aSphere);

  void   readRadiusAndDensity(istream& in);
  void   readRadiusAndWeight(istream& in);
  void   readDensityAndWeight(istream& in);

  static double getRadius(double density, double weight);
  static double getDensity(double radius, double weight);
  static double getWeight(double radius, double density);

 private:
  double myRadius, myDensity, myWeight;
};

inline Sphere::Sphere()
{
  myRadius = myDensity = myWeight = 0.0;
}

inline double Sphere::getRadius() const
{
  return myRadius;
}

inline double Sphere::getDensity() const
{
  return myDensity;
}

inline double Sphere::getWeight() const
{
  return myWeight;
}

const double PI = 3.14159,
             ONE_THIRD = 1.0 / 3.0;

inline void Sphere::setRadiusAndDensity(double newRadius, double newDensity)
{
  if (newRadius > 0.0 && newDensity > 0.0) {
    myRadius = newRadius;
    myDensity = newDensity;
    myWeight = newDensity * 4.0 * PI * pow(newRadius, 3.0) / 3.0;
  }
  else {
    cerr << "setRadiusAndDensity(r,d): r,d must be non-negative" << endl;
  }
}

inline void Sphere::setRadiusAndWeight(double newRadius, double newWeight)
{
  if (newRadius > 0.0 && newWeight > 0.0) {
    myRadius = newRadius;
    myWeight = newWeight;
    myDensity = 3.0 * newWeight / (4.0 * PI * pow(newRadius, 3.0));
  }
  else {
    cerr << "setRadiusAndWeight(r,w): r,w must be non-negative" << endl;
  }
}

inline void Sphere::setDensityAndWeight(double newDensity, double newWeight)
{
  if (newDensity > 0.0 && newWeight > 0.0) {
    myDensity = newDensity;
    myWeight = newWeight;
    myRadius = pow(3.0 * newWeight / (4.0 * PI * newDensity), ONE_THIRD);
  }
  else {
    cerr << "setDensityAndWeight(d,w): d,w must be non-negative" << endl;
  }
}

inline void Sphere::print(ostream & out) const
{
  out << "A sphere of radius " << getRadius()
      << ", density " << getDensity()
      << ", and weight " << getWeight();
}

inline ostream& operator<<(ostream& out, const Sphere& aSphere)
{
  aSphere.print(out);
  return out;
}


inline void Sphere::readRadiusAndDensity(istream& in)
{
  double radius, density;
  in >> radius >> density;
  setRadiusAndDensity(radius, density);
}

inline void Sphere::readRadiusAndWeight(istream& in)
{
  double radius, weight;
  in >> radius >> weight;
  setRadiusAndWeight(radius, weight);
}

inline void Sphere::readDensityAndWeight(istream& in)
{
  double density, weight;
  in >> density >> weight;
  setDensityAndWeight(density, weight);
}


inline double Sphere::getRadius(double density, double weight)
{
  if (density > 0.0 && weight > 0.0)
    return pow(3.0 * weight / (4.0 * PI * density), ONE_THIRD);
  else
    return 0.0;
}

inline double Sphere::getDensity(double radius, double weight)
{
  if (radius > 0.0 && weight > 0.0)
    return 3.0 * weight / (4.0 * PI * pow(radius, 3.0));
  else
    return 0.0;
}

inline double Sphere::getWeight(double radius, double density)
{
  if (radius > 0.0 && density > 0.0)
    return density * 4.0 * PI * pow(radius, 3.0) / 3.0;
  else
    return 0.0;
}

#endif